
;--------------------------------------
; July 6th, 2004
; This file is released into the Public Domain by it's author,
; Jay Summet (jay (at) summet.com). Note that the PIC initalization code
; used as a framework upon which the interrupte driven transmitter
; was built is from the file xmit_demo.asm distributed by
; Microchip Technology Inc under the license below. (the xmit.asm file name
; in the information below is incorrect.)
;
; I would like to thank Intel Research for funding this code development,
; and allowing it's release to the world.
;
;
; This file will run on Microchip RF-PIC transmitter prototype boards as
; found in the rfPic kit. It demonstrates using an interrupt driven transmitter
; which allows you to perform other operations while transmitting data in the
; background. (Note that this particular program doesn't do much other than wait
; for the transmitter to get done, but it COULD be doing something else...)
;

;----------------------------------------------------------------------
;                                                                     
;                     Software License Agreement                      
;                                                                     
; The software supplied herewith by Microchip Technology Incorporated 
; (the "Company") for its PICmicro Microcontroller is intended and   
; supplied to you, the Companys customer, for use solely and         
; exclusively on Microchip PICmicro Microcontroller products.         
;                                                                     
; The software is owned by the Company and/or its supplier, and is     
; protected under applicable copyright laws. All rights are reserved.  
; Any use in violation of the foregoing restrictions may subject the  
; user to criminal sanctions under applicable laws, as well as to     
; civil liability for the breach of the terms and conditions of this  
; license.                                                             
;                                                                      
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,   
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED   
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,   
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR          
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.                    
;                                                                     
;----------------------------------------------------------------------
;
;   Filename:           xmit.asm
;   Date:               April 25, 2003
;   File Version:       1.0
;   Assembled using:    MPASM 03.30 Released
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;
;----------------------------------------------------------------------
;
;   Files required:
;                       p12f675.inc
;
;
;----------------------------------------------------------------------
;
;   Program Description 
;
;   
;; Interrupt driven data transmission! (ick ick ick!)
;
;----------------------------------------------------------------------

    list        p=12f675        ; list directive to define processor
    #include    <p12f675.inc>   ; processor specific variable definitions

    errorlevel  -302            ; suppress message 302 from list file

; ---------------------------------------------------------------------
; Configuration Bits (Section 9.1 Configuration Bits)
; ---------------------------------------------------------------------
;
; Data Memory Code Protection bit:
; _CPD_ON = Enabled
; _CPD_OFF = Disabled
;
; Program Memory Code protection:
; _CP_ON = Enabled
; _CP_OFF = : Disabled
;
; Brown-out Detection Enable bit:
; _BODEN_ON = Enabled
; _BODEN_OFF = Disabled
;
; GP3/MCLR pin function select:
; _MCLRE_ON = GP3/MCLR pin function is /MCLR
; _MCLRE_OFF = GP3/MCLR pin function is digital I/O, 
;              /MCLR internally tied to Vdd
;
; Power-up Timer Enable bit:
; _PWRTE_ON = Enabled
; _PWRTE_OFF = Disabled
;
; Watchdog Timer Enable bit:
; _WDT_ON = Enabled
; _WDT_OFF = Disabled
;
; Oscillator Selction bits:
; _EXTRC_OSC_NOCLKOUT = CLKOUT function on GP4 pin, RC on GP5 pin.
; _EXTRC_OSC_CLKOUT = I/O function on GP4 pin, RC on GP5 pin.
; _INTRC_OSC_CLKOUT =  Internal oscillator, CLKOUT function on GP4 pin,
;                      I/O function on GP5 pin.
; _INTRC_OSC_NOCLKOUT = Internal oscillator, I/O function on GP4 and GP5 pins.
; _EC_OSC = I/O function on GP4 pin, CLKIN on GP5 pin.
; _HS_OSC = High speed crystal/resonator on GP4 and GP5 pins.
; _XT_OSC = Crystal/resonator on GP4 and GP5 pins.
; _LP_OSC = Low power crystal on GP4 and GP5 pins.
;
;
; ---------------------------------------------------------------------

    __CONFIG    _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT

;----------------------------------------------------------------------
; Variables  (Section 2.2 Data Memory Organization)
;----------------------------------------------------------------------

    ; Data Memory Organization (Section 2.2)
    ;
    ; Register locations 0x20 to 0x5F (64 bytes) are General Purpose 
    ; registers, implemented as static RAM and are mapped across both
    ; banks. 

    cblock  0x20

        w_temp                      ; used for context saving 
        status_temp                 ; used for context saving

        TEMP                        ; General Purpose Temporary register

        CSR0                        ; TX buffer shift register
        CSR1
        CSR2
        CSR3
        CSR4
        CSR5
        CSR6
        CSR7
        CSR8

        Count
        Count2
        BitCount
        TimeHi
        TimeLo

        FuncBits                    ; Function Bits

;; Extra Stuff for the Interrupt Driven Transmission!
		TX_STATUS			; 0 = set means currently transmitting
							; 1 = Bit we are currently transmitting.

 	 	TX_STATE			; Stores the state we are in...
		TX_COUNTER			; Used to count a lot of things.
        TX_BIT_COUNTER		; Stores the number of bits we have sent.
		TX_BYTE_TEMP		; Stores the each byte as we send it.
        TX_FSR				; Stores the value of FSR when we are not in the ISR.

 ;; Note. Because our counters never go over 128, TX_BIT_COUNTER and TX_COUNTER 
 ;; Could possible share the same byte (upper/lower nibble) at the expense of 5-10
 ;; extra program words. Also, TX_STATE and TX_STATUS could be combined with another
 ;; 10-15 program words. But, this would make things hairy and difficult to understand,
 ;; so until I need the SRAM I won't do it.

    endc

;; MACRO: Swap W and a register with no extra registers...
SWAPWF	MACRO REG
		xorwf REG,F
		xorwf REG,W
		xorwf REG,F
		ENDM

;----------------------------------------------------------------------
; Defines
;----------------------------------------------------------------------

    ; Set up GPIO Port (Section 3.0)
    ; Function of GPIO pins depend on:
    ;   Configuration Bits (CONFIG) (Section 9.1)
    ;   Weak Pull-up Register (WPU) (Section 3.2.1)
    ;   Interrupt-on-change GPIO Register (IOCB) (Section 3.2.2)
    ;   Option Register (OPTION_REG) (Register 4-1)
    ;   TIMER1 Control Register (T1CON) (Register 5-1)
    ;   Comparator Control Register (CMCON) (Section 6.0)
    ;   A/D Control Register (ADCON0) (Section 7.0) (PIC12F675 Only)

#define POT0    GPIO, 0             ; (Analog Input) Potentiometer GP0
#define POT1    GPIO, 1             ; (Analog Input) Potentiometer GP1
#define TXD     GPIO, 2             ; (Output) Transmit Data
#define PB3     GPIO, 3             ; (Input Only) Push button switch GP3
#define PB4     GPIO, 4             ; (Input) Push button switch GP4
#define RFENA   GPIO, 5             ; (Output) RF Enable

    ; Define for TRISIO Register (Section 3.1)
    ;
    ; GPIO is an 6-bit wide, bi-directional port. The corresponding data
    ; direction register is TRISIO. Setting a TRISIO bit (= 1) will make
    ; the corresponding GPIO pin and input. Clearing a TRISIO bit (= 0)
    ; will make the corresponding GPIO pin an output. The exception is
    ; GP3, which is input only and its TRIS bit will always read as a '1'.

    ; GPIO Pins = xx543210
#define GPTRIS  B'00011011'

    ; delays/timings

#define TGUARD  D'46'               ; 46 X TE
#define TSYNC   D'10'				; 10 x TE
#define PREAMB  D'16'               ; Preamble length = 16 pulses


;----------------------------------------------------------------------
; Program Memory
;----------------------------------------------------------------------

    ; Program Memory Organization (Section 2.1)

        ORG     0x000               ; RESET Vector

        nop                         ; for ICD use
        goto    INITALIZE           ; goto INITALIZE


        ORG     0x004               ; Interrupt Vector

        movwf   w_temp              ; save W register
        swapf   STATUS, W           ; swap status to be saved into W
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movwf   status_temp         ; save STATUS register

;----------------------------------------
; Interrupt Service Routine (ISR)
;
; Description: 
;   
;----------------------------------------

; Interrupt-on-change (Section 3.2.2 and 9.4.3)
;
; An input change on GPIO change sets the GPIF bit. The interrupt can
; be enabled/disabled be setting/clearing the GPIE bit. Individual pins
; are configured through the IOC register (see INITIALIZATION below).
;
; Clear the IOC interrupt by:
;  a) Any read or write of GPIO. This will end the mismatch condition.
;  b) Clear the flag bit GPIF

        movfw   GPIO                ; read GPIO
        bcf     INTCON, GPIF
;----------------------------------------
;; If this is a Timer0 interrupt, call the
;; Interrupt Driven TX transmission handler...
     ;movf INTCON,W
     ;andlw 0x20
     ;btfsc STATUS,Z				;; IS T0IE set?
     ;goto RETURN_FROM_ISR		;; no, continue.
     movf INTCON,W				;; Check T0IF
     andlw 0x04		
     btfsc STATUS,Z				;; If T0IF is set, handle it!
      goto RETURN_FROM_ISR

	

	;; If we are inactive we set this to zero...
    clrw  ;; Load zero into w, to put into TMR0 unless we are active.
    btfsc TX_STATUS,0
    movlw D'215'		;; If we are active, set TMR0 to 210 = 49 x 8 cycles = 400
    movwf TMR0

     btfsc TX_STATUS,0			;; Check to see if we are active.
     call TX_INTERRUPT			;; Go do the real work!


     bsf     STATUS, RP0         ; ---- Select Bank 1 -----
     bcf INTCON,T0IF			;; Clear the TMR0 interrupt flag.
     bcf     STATUS, RP0         ; ---- Select Bank 0 -----
 ;----------------------------------------
RETURN_FROM_ISR
        swapf   status_temp, W      ; swap status_temp into W, sets bank to original state
        movwf   STATUS              ; restore STATUS register
        swapf   w_temp, F
        swapf   w_temp, W           ; restore W register

        retfie


;----------------------------------------------------------------------
; Subroutine DATA_EEPROM_READ
;
; Description: To read an EEPROM data memory location, the address is
;   written to the EEADR register and set control bit RD (EECON1<0>) to
;   initiate a read. Data is available in the EEDATA register the next
;   clock cycle.
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: W contains EEPROM address to be read
;   
; Output: W contains EEPROM data
;   
;----------------------------------------------------------------------

DATA_EEPROM_READ

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        movwf   EEADR               ; move EEPROM address in W to EEADR
        bsf     EECON1, RD          ; initiate EEPROM read
        movf    EEDATA, W           ; move data to W

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return


;----------------------------------------------------------------------
; Subroutine DATA_EEPROM_WRITE
;
; Description: To write an EEPROM data memory location, the address is
;   written to the EEADR register, data to the EEDATA register, then
;   execute a required sequence of instructions.
;   
; CAUTION: Interrupts are disable during this subroutine
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: Address = EEADR, Data = EEDATA
;   
; Output: none
;   
;----------------------------------------------------------------------

DATA_EEPROM_WRITE

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bsf     EECON1, WREN        ; EEPROM Write Enable: allow write cycles
        bcf     INTCON, GIE         ; disable global interrupts
                                    ; *** required sequence, do not alter ***
        movlw   0x55
        movwf   EECON2
        movlw   0xAA
        movwf   EECON2
        bsf     EECON1, WR          ; initiate EEPROM write
                                    ; *** end required sequence ***


        bsf     INTCON, GIE         ; enable global interrupts
        bcf     EECON1, WREN        ; EEPROM Write Enable: inhibit write cycles

;; Wait for the write to finish!
DEW_WAIT_LOOP
        btfsc   EECON1,WR			; Querry the "I'm done" bit... 
		goto DEW_WAIT_LOOP			; loop until it's cleared by hardware... (yes, it's a waste of cycles, but hey...)

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return


;----------------------------------------------------------------------
; Subroutine READ_ANALOG_AN0; READ_ANALOG_AN1
;
; Description: Read analog channel 0 (AN0) or 1 (AN1).
;   
; Constants: none
;   
; Global Variables: none
;   
; Initialization: none
;   
; Output: ADRESH and ADRESL contain 10-bit A/D result justified 
;   according to ADCON0, ADFM bit.
;   
;----------------------------------------------------------------------

READ_ANALOG_AN0

        bcf     ADCON0, CHS1        ; select analog channel AN0
        bcf     ADCON0, CHS0

        goto    READ_ANALOG

READ_ANALOG_AN1

        bcf     ADCON0, CHS1        ; select analog channel AN1
        bsf     ADCON0, CHS0

READ_ANALOG

        bsf     ADCON0, ADON        ; Turn on ADC module

        ; After selecting a new channel, allow for sufficent sample time.
        ; The amount of sample time depends on the charging time of the
        ; internal charge holding capacitor (Section 7.2).

        movlw   D'6'                ; At 4 MHz, a 22us delay
        movwf   TEMP                ; (22us = 2us + 6 * 3us + 1us)
        decfsz  TEMP, F
        goto    $-1

        bsf     ADCON0, GO          ; start A/D conversion

        btfsc   ADCON0, GO          ; has A/D conversion completed?
        goto    $-1

        bcf     ADCON0, ADON        ; Turn off ADC module (consumes no operating current)

        return


;----------------------------------------------------------------------
; Subroutine: WaitxTE
;   
; Description: 
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: W x 400us length of delay required
;   
; Output: Count, Count2   as decrementing counters
;   
;----------------------------------------------------------------------

WaitxTE
        movwf   Count2      ; [1]

waitxlp            
        movlw   D'79'       ; [1]
        movwf   Count       ; [1]

wait400lp
        nop                 ; [1]
        nop                 ; [1]
        decfsz  Count,F     ; [1]
         goto    wait400lp  ; [2]
;                          --------
;                         79 x 5 = 395us

        decfsz  Count2,F    ; [1]
         goto    waitxlp    ; [2]

        retlw   0           ; [2]

;                       total 2 (call) + W x (395 + 5) + 2 (return)
;                       w = 1 -> 406us @4MHz
;                       w = 2 -> 806us @4MHz


;----------------------------------------------------------------------
; Initialize PICmicro (PIC12F675)
;----------------------------------------------------------------------

INITALIZE

; Disable global interrupts during initialization

        bcf     INTCON, GIE         ; disable global interrupts


;----------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (Section 2.2.2.7)
;
; A calibration instruction is programmed into the last location of
; program memory. This instruction is a RETLW XX, where the literal is
; the calibration value. The literal is placed in the OSCCAL register
; to set the calibration of the internal oscillator.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        call    0x3FF               ; retrieve factory calibration value
        movwf   OSCCAL              ; update register with factory cal value

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; GPIO Port (Section 3.0)
;
; Store GPTRIS value defined above into the TRISIO direction register

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        movlw   GPTRIS
        movwf   TRISIO              ; Write to TRISIO register

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Comparator Module (Section 6.0)
;
; The PIC12F629/675 devices have one analog comparator. The inputs to
; the comparator are multiplexed with the GP0 and GP1 pins. There is 
; an on-chip Comparator Voltage Reference that can also be applied to
; an input of the comparator. In addition, GP2 can be configured as
; the comparator output. The Comparator Control Register (CMCON)
; contains bits to control the comparator. The Voltage Reference
; Control Register (VRCON) controls the voltage reference module.


        ; Comparator Configuration (Figure 6-2)
;        bcf     CMCON, CINV         ; Comparator Output Inversion: not inverted
;        bcf     CMCON, COUT         ; Comparator Output bit: Vin+ < Vin-
;        bcf     CMCON, CIS          ; Comparator Input Switch: Vin- connectos to Cin-

        ; CM2:CM0 = 111 - Comparator Off (lowest power)
        bsf     CMCON, CM2          ; Comparator Mode bit 2
        bsf     CMCON, CM1          ; Comparator Mode bit 1
        bsf     CMCON, CM0          ; Comparator Mode bit 0

        ; VRCON (Register 6-2)
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     VRCON, VREN         ; CVref circuit: powered down, no Idd drain

;        bcf     VRCON, VRR          ; CVref Range Selection: High Range

;        bcf     VRCON, VR3          ; CVref value selection bit 3
;        bcf     VRCON, VR2          ; CVref value selection bit 2
;        bcf     VRCON, VR1          ; CVref value selection bit 1
;        bcf     VRCON, VR0          ; CVref value selection bit 0

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Analog-to-Digital Converter (A/D) Module (Section 7.0) (PIC12F675 Only)
;
; The analog-to-digital converter (A/D) allows conversion of an analog
; input signal to a 10-bit binary representation of that signal. The
; PIC12F675 has four analog inputs multiplexed into one sample and hold
; circuit. There are two registers to control the functions of the A/D
; module:
;   A/D Control Register (ADCON0)
;   Analog Select Register (ANSEL)
;
; Note: When using GPIO pins as analog inputs, ensure the TRISIO register
;       bits are set (= 1) for input.

        bcf     ADCON0, ADFM        ; A/D Result Formed: left justified
        bcf     ADCON0, VCFG        ; Voltage Reference: Vdd
        bcf     ADCON0, ADON        ; ADC is shut-off and consumes no operating current

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        ; select A/D Conversion Clock Source: Fosc/8
        bcf     ANSEL, ADCS2        ; A/D Conversion Clock Select bit 2
        bcf     ANSEL, ADCS1        ; A/D Conversion Clock Select bit 1
        bsf     ANSEL, ADCS0        ; A/D Conversion Clock Select bit 0

        ; select GPIO pins that will be analog inputs: GP0/AN0, GP1/AN1
        bcf     ANSEL, ANS3         ; Analog Select GP4/AN3: digital I/O
        bcf     ANSEL, ANS2         ; Analog Select GP2/AN2: digital I/O
        bsf     ANSEL, ANS1         ; Analog Select GP1/AN1: analog input
        bsf     ANSEL, ANS0         ; Analog Select GP0/AN0: analog input

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; TIMER1 Module with Gate Control (Section 5.0)
;
; The TIMER1 Control Register (T1CON) is used to enable/disable TIMER1
; and select various features of the TIMER1 module.

        bcf     T1CON, TMR1ON       ; TIMER1: stopped

        bcf     T1CON, TMR1CS       ; TIMER1 Clock Source Select: Internal Clock (Fosc/4)

        bcf     T1CON, NOT_T1SYNC   ; TIMER1 External Clock Input Sync Control: Syncronize external clock input

        ; T1OSCEN only if INTOSC without CLKOUT oscillator is active, else ignored
        bcf     T1CON, T1OSCEN      ; LP Oscillator Enable Control: LP oscillator off

        ; TIMER1 Input Prescale Select: 1:1
        bcf     T1CON, T1CKPS1      ; TIMER1 Input Clock Prescale Select bit 1
        bcf     T1CON, T1CKPS0      ; TIMER1 Input Clock Prescale Select bit 0

        ; TMR1GE only if TMR1ON = 1, else ignored
        bcf     T1CON, TMR1GE       ; TIMER1 Gate Enable: on


;----------------------------------------
; Weak Pull-up Register (WPU) (Section 3.2.1)
;
; Each of the GPIO pins, except GP3, has an individually configurable
; weak internal pull-up. Control bits WPUx enable or disable each 
; pull-up. Refer to Register 3-1. Each weak pull-up is automatically
; turned off when the port pin is configured as an output. The pull-ups
; are disabled on a Power-on Reset by the NOT_GPPU bit (see OPTION_REG below).

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

;     GPIO Pins = xx54x210
        movlw   B'00010000'         ; GP4 pull-up enabled
        movwf   WPU

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; OPTION Register (OPTION_REG) (Section 2.2.2.2)
; TIMER0 Module (Section 4.0)
;
; The OPTION_REG contains control bits to configure:
;   Weak pull-ups on GPIO (see also WPU Register above)
;   External GP2/INT interrupt
;   TMR0
;   TMR0/WDT prescaler

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     OPTION_REG, NOT_GPPU  ; GPIO pull-ups: enabled

        bsf     OPTION_REG, INTEDG  ; Interrupt Edge: on rising edge of GP2/INT pin

        bcf     OPTION_REG, T0CS    ; TMR0 Clock Source: internal instruction cycle (CLKOUT)
        bcf     OPTION_REG, T0SE    ; TMR0 Source Edge: increment low-to-high transition on GP2/T0CKI pin

        bcf     OPTION_REG, PSA     ; Prescaler Assignment: assigned to TIMER0

        ; TMR0 Prescaler Rate: 1:8
        bcf     OPTION_REG, PS2     ; Prescaler Rate Select bit 2
        bsf     OPTION_REG, PS1     ; Prescaler Rate Select bit 1
        bcf     OPTION_REG, PS0     ; Prescaler Rate Select bit 0

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Interrupt-on-Change Register (IOCB) (Section 3.2.2)
;
; Each of the GPIO pins is individually configurable as an interrupt-
; on-change pin. Control bits IOCBx enable or disable the interrupt 
; function for each pin. Refer to Register 3-2. The interrupt-on-change
; is disabled on a Power-on Reset.
;
; Note: Global interrupt enables (GIE and GPIE) must be enabled for
;       individual interrupts to be recognized.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

    ; GPIO Pins = xx543210
        movlw   b'00011000'
        movwf   IOCB                ; Interrupt-on-change enabled: GP3, GP4

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Peripheral Interrupt Enable Register (PIE1) (Section 2.2.2.4)
;
; The PIE1 register contains peripheral interrupt enable bits.
;
; Note: The PEIE bit (INTCON<6>) must be set to enable any 
;       peripheral interrupt.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     PIE1, EEIE          ; EE Write Complete Interrupt: disabled
        bcf     PIE1, ADIE          ; A/D Converter Interrupt (PIC12F675 Only): disabled
        bcf     PIE1, CMIE          ; Comparator Interrupt: disabled
        bcf     PIE1, TMR1IE        ; TMR1 Overflow Interrupt: disabled

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Interrupt Control Register (INTCON) (Section 2.2.2.3)
;
; The INTCON register contains enable and disable flag bits for TMR0
; register overflow, GPIO port change and external GP2/INT pin 
; interrupts.

        bcf     INTCON, PEIE        ; disable Peripheral Interrupt Enable bit
        bsf     INTCON, T0IE        ; Enable TMR0 Overflow Interrupt Enable bit
;	 bcf INTCON,T0IE   ; FOR TESTING, disable interrupt.
        bcf     INTCON, INTE        ; disable GP2/INT External Interrupt Enable bit
        bsf     INTCON, GPIE        ; enable Port Change Interrupt Enable bit

;        bcf     INTCON, GIE         ; disable global interrupts


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Main Program
;----------------------------------------------------------------------
;----------------------------------------------------------------------

MAIN
		clrf TX_STATE			; Start with TX_State at zero...
        bcf     RFENA           ; Disable Transmitter
        bsf     INTCON, GIE     ; enable global interrupts

;----------------------------------
; Scan push buttons
;----------------------------------

SCANPB

        movlw   0x00            ; load zero into W
        movwf   FuncBits        ; clear Function Bits register

        btfsc   PB3             ; Push Button GP3 pressed?
         goto   SPB1
        movlw   0x23            ; Function S0 selected
        iorwf   FuncBits, F
        call    READ_ANALOG_AN0 ; read analog channel AN0

SPB1
        btfsc   PB4             ; Push Button GP4 depressed?
         goto   SPB2            ; no, jump over
        movlw   0x43            ; Function S1 selected
        iorwf   FuncBits, F
        call    READ_ANALOG_AN1 ; read analog channel AN1

SPB2
        movlw   0xFF
        andwf   FuncBits, W     ; Was any switch depressed?
        btfss   STATUS, Z
         goto    XMIT            ; Yes, transmit buffer


;; Before we go to sleep, make sure that the Transmitter is done sending...
FT1_LOOP_UNTIL_DONE
	;; Now, we wait until the interrupt driven transmitter finishes
    ;; things up, including clearing the TX_STATUS,0 bit...
    btfsc TX_STATUS,0
     goto FT1_LOOP_UNTIL_DONE

        sleep                   ; Put PICmicro to sleep
        goto    SCANPB          ; Upon wake-up on pin change, scan push buttons


;----------------------------------
; fill in transmission buffer
;----------------------------------

XMIT

	;; If we are currently transmitting, wait here until we are done...

FT_LOOP_UNTIL_DONE
	;; Now, we wait until the interrupt driven transmitter finishes
    ;; things up, including clearing the TX_STATUS,0 bit...
    btfsc TX_STATUS,0
     goto FT_LOOP_UNTIL_DONE

       
	 	;; Load the data into the CSR registers!
        movlw   0x73            ; send Serial number  
        movwf   CSR0

        movfw   FuncBits        ; send Function Bits
        movwf   CSR1

                                ; send 16-bit Counter
                                ;
                                ; send analog value (this is modified from the fixed 
                                ;   KeeLoq(r) protocol; these two fields are normally 
                                ;   used for 16-bit counter value)

        bsf     STATUS, RP0     ; ----- Select Bank 1 -----
        movfw   ADRESL          ; ADRESL Result
        bcf     STATUS, RP0     ; ----- Select Bank 0 -----
        movwf   CSR2

        movfw   ADRESH          ; ADRESH Result
        movwf   CSR3

        movlw   0x56            ; send 32-bit serial number 1 (ignored by receiver)
        movwf   CSR4

        movlw   0x34
        movwf   CSR5

        movlw   0x12
        movwf   CSR6

        movlw   0x20
        movwf   CSR7

        movlw   0x55            ; send Flags
        movwf   CSR8


;; Start the interrupt driven Transmitter!

 movlw D'1' ;; State = 1 TX_START_PREAMBLE
 movwf TX_STATE
 bsf TX_STATUS,0	;; Start transmitting.






        goto    SCANPB          ; go back and scan for push button presses and things while
								; the transmitter is running in the background...








;----------------------------------------------------------------------
; TX_INTERRUPT
;
;  This is the main interrupt handler for Interupt based transmission.
;  Depending upon the TX_STATE variable it will shuffle controll
;  off to the various parts of the interrupt driven TX state machine.

TX_INTERRUPT
 ;  bcf TX_STATUS,0
 ;  return
    movlw high TX_STATE_MACHINE_START
    movwf PCLATH
    movf  TX_STATE,W
    andlw 0x0F;			;; May need to CHANGE ME if more than 16 states (below)!
    addlw low TX_STATE_MACHINE_START
    btfsc STATUS,C
    incf PCLATH,F
    movwf PCL

							;; load PC with computed goto value.

TX_STATE_MACHINE_START
 goto TX_RADIO_INACTIVE				;0
 goto TX_PREAMBLE_START				;1
 goto TX_PREAMBLE_ON				;2
 goto TX_PREAMBLE_OFF				;3
 goto TX_SYNC_TIME_START			;4
 goto TX_SYNC_TIME_WAIT				;5	
 goto TX_NEXT_BYTE					;6
 goto TX_NEXT_BIT					;7
 goto TX_TRANSITION_ONE				;8
 goto TX_END_BIT					;9
 goto TX_GUARD_TIME_START			;10	
 goto TX_GUARD_TIME_WAIT			;11
 goto TX_FINISHED					;12


;------------------
; TX_RADIO_INACTIVE
; 
; We are not currently transmitting. As a matter of fact, we shouldn't reach
; this state if the TX_STATUS bit is set correctly...
TX_RADIO_INACTIVE
  ;; We really shouldn't get to this position...
  bcf TX_STATUS,0
  clrf TMR0
  return
;-----------------
; TX_PREAMBLE_START
;
; This is the PREAMBLE Init. So we load TPREAMBLE into the TX_COUNTER
; (number of interrupt periods we should toggle) and then increment the 
; state to TX_PREAMBLE_ON 
TX_PREAMBLE_START

  bsf     RFENA           ; Enable Transmitter

  movlw  PREAMB
  movwf TX_COUNTER
  incf TX_STATE,F
  return

;------------------
; TX_PREAMBLE_ON
;
; Turn on the transmitter for one bit, then advance to TX_PREAMBLE_OFF
;
TX_PREAMBLE_ON
   bsf     TXD 				; ON
   incf TX_STATE,F			; Move on to TX_PREAMBLE_OFF next...
   return

;--------------------
; TX_PREAMBLE_OFF
;
; Turn the transmitter off for one bit. Then, Decrement the TX_COUNTER
; and (if not zero) decrement back to TX_PREAMBLE_ON.
; Basically we are flashing an on-off pattern PREAMB times.  
TX_PREAMBLE_OFF
 bcf TXD					; OFF
 decf TX_COUNTER,F
 btfsc STATUS,Z
 goto TX_PREAMBLE_OFF_DONE
 decf TX_STATE,F			; Not done yet, go back a state to TX_PREAMBLE_ON
 return

;; We are done with the preamble, radio is turned off, go to the TX_SYNC_TIME_START state
TX_PREAMBLE_OFF_DONE

 incf TX_STATE,F
 return

;-----------------
; TX_SYNC_TIME_START
;
; This is the SYNC time. So we load TSYNC into the TX_COUNTER
; (number of interrupt periods we should wait) and then increment the state
; to TX_SYNC_TIME_WAIT. Note that the radio is turned off now (by previous state)
TX_SYNC_TIME_START
  movlw  TSYNC
  movwf TX_COUNTER
  incf TX_STATE,F
  return

;------------------
;TX_SYNC_TIME_WAIT
;
; We are currently waiting on the SYNC time. One interrupt has
; occured, so we decrement the TX_COUNTER. If it reaches zero,
; we are done waiting, so we change the STATE to TX_FINISHED.
TX_SYNC_TIME_WAIT
 decf TX_COUNTER,F
 btfss STATUS,Z
 return

 ;; We are done with the Sync, now we get ready to transmit!
 
  movlw D'8'		; Send 64 bits in total, 8 bytes.
  movwf TX_COUNTER

 ;; Load our indirection value for FSR into TX_FSR
 movlw CSR0
 movwf TX_FSR

 
 incf TX_STATE,F		;; Move on to TX_NEXT_BYTE
 return

;-------------------
; TX_NEXT_BYTE
;
; Get the next byte ready to send.
;
TX_NEXT_BYTE

  ;; We turn on the transmitter right now! 
  ;; (I.E. this code runs inside of the transmition of the first "ON" phase.
  ;;   as we roll over into TX_NEXT_BIT.
  bsf TXD

 ;; Save the FSR register (so we don't mess up people outside of the ISR)
 ;; At the same time, move TX_FSR into FSR.
  SWAPWF FSR			; Note that the macro makes this 9 instructions total!
  SWAPWF TX_FSR		; 3 for each swap.
  SWAPWF FSR

 ; Move the current byte to send into TX_BYTE_TEMP.
  movfw INDF
  movwf TX_BYTE_TEMP		

 ;; Swap the FSR back out.
  SWAPWF FSR
  SWAPWF TX_FSR
  SWAPWF FSR
 
  movlw D'8'			; 8 bits in a byte.
  movwf TX_BIT_COUNTER


  incf TX_STATE,F	; We are moving into TX_NEXT_BIT, so we have to increment this,
					;; Even if we don't loop through another interupt.
  ;return

	;; We now move on to TX_NEXT_BIT. Note that we do NOT pause for another interrupt here!
    ;; So, the transmitter turns on almost immediately (with the briefest of pauses to
    ;; increment to the next byte...


;-----------------
; TX_NEXT_BIT
;
; Determine the next bit to send, and set the TX_STATUS.1 bit appropriately.
; Then, turn on the transmitter and  advance to TX_TRANSITION_ONE
;
TX_NEXT_BIT
  ;; First. Turn on the transmitter (to get it on as early as possible.)
  ;; This only matters if we are being called directly (as opposed to falling
  ;; through from TX_NEXT_BYTE.
  bsf TXD

  bcf TX_STATUS,1			;; Bit that holds the value of the current bit to save...
							;; We start off assuming our bit is a zero...

  rrf TX_BYTE_TEMP,F		;; Move the bit to send into carry.
  btfsc STATUS,C			;; test it...
  bsf TX_STATUS,1			;; If our bit is a one, set TX_STATUS.1


  incf TX_STATE,F		;; Move to TX_TRANSITION_ONE
  return


;; How does this next part work you ask? Easy...
;ZERO            
; 
;   +---+---+   +--
;   |       |   |
;---+       +---+
;   |  2Te   Te |
;
; If we are sending a zero (clear) we leave the transmitter on until TX_END_BIT.

;ONE
;              
;   +---+       +--
;   |   |       |
;---+   +---+---+
;   | Te   2Te  |
; 
; If we are sending a one (set) we turn the transmitter off early, at TX_TRANSITION_ONE
;
; The receiver checks the time on vs the time off and figures out what bit we are sending... easy as pie.
;
;
; To Clarify how the states of the machine interact with the above pattern, look at this:
;
;   +---+       +--
;   |   |       |
;---+   +---+---+
;   1   2   3   1  2....
;   +---+---+   +--
;   |       |   |
;---+       +---+
;   |  2Te   Te |
;
; TX_NEXT_BYTE and TX_NEXT_BIT run at point 1.
; TX_TRANSITION_ONE is at position 2.
; TX_END_BIT runs at point 3.
;
; So, after TX_NEXT_BYTE and TX_NEXT_BIT get done, the transmitter is turned on until
; at least point 2. At point 2, TX_TRANSITION_ONE gets ran. If we are transmitting a one, 
; we turn the transmitter off here. (If a zero, we don't.)
;
; Then, at point 3 TX_END_BIT gets called. It turns the transmitter off (again in the case of a one)
; and then sets things up to jump back to either TX_NEXT_BIT or TX_NEXT_BYTE (which happen at point 1
; of the next bit or byte. And the cycle continues.

;----------------
; TX_TRANSITION_ONE
;
; We are not at point 2 in the above ascii art figures...
; If we are sending a ONE, we turn off the transmitter. If we are sending
; a zero, we leave it on (to be turned off at TRANSITION_TWO).
;
TX_TRANSITION_ONE

  btfsc TX_STATUS,1		;; Test to see what bit we are supposed to be sending.
	bcf TXD				;; If it's a 1 (SET) we turn off the transmitter.


  incf TX_STATE,F		; Move on to TX_END_BIT
  return

;---------------------
; TX_END_BIT
;
; We are now at point 3 in the above ascii art pictures..
;
; We decrement the bit sent count. If it's not zero, we go back to
; TX_NEXT_BIT. If it is zero, we are done with this byte, so we decremnt
; TX_COUNTER. If TX_COUNTER is not zero, we inc the TX_FSR and jump to TX_NEXT_BYTE. If it is zero,
; it means we are done, and we jump to TX_GUARD_TIME_START.
TX_END_BIT

  ;; Doesn't really matter what the bit is set to, once we get here we want
  ;; the transmitter off no matter what...
  bcf TXD

  decf TX_BIT_COUNTER,F
  btfss STATUS,Z
  goto _TX_END_BIT_NEXT_BIT ;; Bit counter is not zero, so we do the next bit.
 
  ;; Bit counter was zero, so we are done with the current byte.
 
  decf TX_COUNTER,F
  btfsc STATUS,Z			;; Check to see if we are all done..
  goto _TX_END_BIT_DONE		;; Ok, we are done sending all the bytes 

 ;; Not all done, so we go back to TX_NEXT_BYTE
_TX_END_BIT_NEXT_BYTE
  incf TX_FSR,F			;; Move on to the next byte!

  movlw D'6'			; TX_NEXT_BYTE
  movwf TX_STATE
  return

 ;; We need to send the next bit, so we jump to TX_NEXT_BIT
_TX_END_BIT_NEXT_BIT
 movlw D'7' 		; TX_NEXT_BIT
 movwf TX_STATE
 return

 ;; We are all done sending bits, so we increment to TX_GUARD_TIME_START
_TX_END_BIT_DONE
 incf TX_STATE,F
 return




;-----------------
; TX_GUARD_TIME_START
;
; It is time to start the guard time. So we load TGUARD into the TX_COUNTER
; (number of interrupt periods we should wait) and then increment the state
; to TX_GUARD_TIME_WAIT. Note that the radio is turned off now (by previous state)
TX_GUARD_TIME_START
  movlw  TGUARD
  movwf TX_COUNTER
  incf TX_STATE,F
  return

;------------------
;TX_GUARD_TIME_WAIT
;
; We are currently waiting on the guard time. One interrupt has
; occured, so we decrement the TX_COUNTER. If it reaches zero,
; we are done waiting, so we increment the STATE to TX_FINISHED.
TX_GUARD_TIME_WAIT
 decf TX_COUNTER,F
 btfss STATUS,Z
 return
 ;; We are done, so increment state!
 incf TX_STATE,F
 return

;--------------------
; TX_FINISHED
;
; We are done with this transmission. Do all of the cleanup, and then
; clear the state register back to TX_RADIO_INACTIVE.
TX_FINISHED
  bcf     RFENA              ; Shut down the Transmitter
  clrf TX_STATE				;; Next state is TX_RADIO_INACTIVE
  bcf TX_STATUS,0			;; Clear the "currently transmitting" flag
  return

;----------------------------------------------------------------------
; Data EEPROM Memory (Section 8.0)
;
; PIC12F629/675 devices have 128 bytes of data EEPROM with address
; range 0x00 to 0x7F.

        ; Initialize Data EEPROM Memory locations

        ORG 0x2100
        DE  0x73, 0x01, 0x02, 0x03


;----------------------------------------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (OSCCAL) (Section 2.2.2.7)
;
; The below statements are placed here so that the program can be
; simulated with MPLAB SIM.  The programmer (PICkit or PROMATE II)
; will save the actual OSCCAL value in the device and restore it.
; The value below WILL NOT be programmed into the device.

        org     0x3ff
        retlw    0x80

;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
