
/* StaticTracker.cpp
 *  - Defines the simplist tracker we can possibly make.
 *   (e.g. the sensor never moves from it's last position)
 *   We basically reposition the tracker to the last observation,
 *   and do not adjust the size. 
 */
#include <iostream>	// for NULL
#include "StaticTracker.h"




StaticTracker::StaticTracker(float maxX, float maxY) : Tracker(maxX, maxY)
{
 // We don't do anything different from the default, this is just here so
 // that the default constructer gets called with the appropriate arguments.
 
}

  // When the system is ready to put up a new tracker, it will
  // ask the Tracker object where it should be placed.
  // The "timestep" is the current timestep, while the X,Y,Size
  // pointers are used as output variables to return the answer.
  //
  // The return value is a measure of how well we think we are doing.
  // This number drops to zero as we loose the sensor, and goes towards
  // 1.0 as we track it.
  //
float StaticTracker::getTrackerPosition(int timestep, float *X, float *Y, float *size)
{

	// get the last location!
	h_ent * last = getHistory(0);

	// If there is no last location, return zero confidence.
	if (last == NULL)
		return(0.0);

	*X = last->x;
	*Y = last->y;
	*size = DEFAULT_TRACKER_SIZE;

	// Figure out how long it's  been since we last saw something.
	int difference = timestep - last->timestep;

	//diag.	
	//printf("difference is: %d\n",difference);

	// If it's been over a two seconds, we've lost it!
	if (difference > 2000)
		return(0.0);

	// If it's smaller than 2 seconds, return a value based upon
	// the time.
	return( ((float)(2000 - difference)) / 2000.0);

}



