
/* StaticTrackerVS.cpp
 *   We basically reposition the tracker to the last observation,
 *   but we also reposition it's size depending upon the last few
 *   observations. So, for example, when the sensor first moves, we
 *   grow a bit (and keep growing if it is still moving) otherwise
 *   we shrink back down as quickly as possible...
 */
#include <iostream>	// for NULL
#include "StaticTrackerVS.h"




StaticTrackerVS::StaticTrackerVS(float maxX, float maxY) : Tracker(maxX, maxY)
{
 // We don't do anything different from the default, this is just here so
 // that the default constructer gets called with the appropriate arguments.
 
}

  // When the system is ready to put up a new tracker, it will
  // ask the Tracker object where it should be placed.
  // The "timestep" is the current timestep, while the X,Y,Size
  // pointers are the current tracker values and also 
  // used as output variables to return the new tracker values. 
  //
  // The return value is a measure of how well we think we are doing.
  // This number drops to zero as we loose the sensor, and goes towards
  // 1.0 as we track it.
  //
float StaticTrackerVS::getTrackerPosition(int timestep, float *X, float *Y, float *size)
{

	// get the last location!
	h_ent * last = getHistory(0);

	// If there is no last location, return zero confidence.
	if (last == NULL)
		return(0.0);

	*X = last->x;
	*Y = last->y;

	if (getNumHistory() < 3)
	{
	*size = DEFAULT_TRACKER_SIZE;
	}
	else
	{
	 // What is the average time between (the last 3) samples?
	 int average = getAverageSampleTimeDifference(3);

	 // What is the time difference from "Now" to the last sample?
         int lastSampleSeen = timestep - last->timestep;

	 // diag:
	 //printf("average TD: %d  Last TD: %d\n",average,lastSampleSeen);

	 // If we have "lost" a sample for quite a while, it's time to grow!
	 if ((average * 3) < lastSampleSeen)
	 {
		 if (*size < DEFAULT_TRACKER_SIZE)
			 *size = DEFAULT_TRACKER_SIZE;
		 *size *= 1.5;

		 if (*size > MAX_TRACKER_SIZE)
			 *size = MAX_TRACKER_SIZE;
	 }

         // If we are getting regular samples, it's time to shrink...
	 if ( ((float)average  * 1.1) > lastSampleSeen)
		 *size *= 0.75;
	 	if (*size < MIN_TRACKER_SIZE)
			*size = MIN_TRACKER_SIZE;
	}

	
	// Figure out how long it's  been since we last saw something.
	int difference = timestep - last->timestep;

	//diag.	
	//printf("difference is: %d\n",difference);

	// If it's been over a 2.5 seconds, we've lost it!
	if (difference > 2500)
	{
		return(0.0);
	}

	// If it's smaller than 2.5 seconds, return a value based upon
	// the time.
	return( ((float)(2500 - difference)) / 2500.0);

}



